==> krb4-0.0.tar.gz: krb4-0.0/src/lib/krb/get_admhst.c <==
/*
 * This software may now be redistributed outside the US.
 *
 * $Source: /afs/nada.kth.se/home/staff/bg/Work/krb4-0.0/src/lib/krb/RCS/get_admhst.c,v $
 *
 * $Locker:  $
 */

/* 
  Copyright (C) 1989 by the Massachusetts Institute of Technology

   Export of this software from the United States of America is assumed
   to require a specific license from the United States Government.
   It is the responsibility of any person or organization contemplating
   export to obtain such a license before exporting.

WITHIN THAT CONSTRAINT, permission to use, copy, modify, and
distribute this software and its documentation for any purpose and
without fee is hereby granted, provided that the above copyright
notice appear in all copies and that both that copyright notice and
this permission notice appear in supporting documentation, and that
the name of M.I.T. not be used in advertising or publicity pertaining
to distribution of the software without specific, written prior
permission.  M.I.T. makes no representations about the suitability of
this software for any purpose.  It is provided "as is" without express
or implied warranty.

  */

/*
 *
 * $Log: get_admhst.c,v $
 * Revision 1.3  1995/08/10 20:06:17  bg
 * Check if kerberos.REALMNAME server exist in the DNS.
 *
 * Revision 1.2  1995/08/10 13:35:04  bg
 * Use multiple configuration directories.
 *
 * Revision 1.1  1995/08/10 12:15:00  bg
 * Initial revision
 *
 *
 */

#include "krb_locl.h"

/*
 * Given a Kerberos realm, find a host on which the Kerberos database
 * administration server can be found.
 *
 * krb_get_admhst takes a pointer to be filled in, a pointer to the name
 * of the realm for which a server is desired, and an integer n, and
 * returns (in h) the nth administrative host entry from the configuration
 * file (KRB_CONF, defined in "krb.h") associated with the specified realm.
 *
 * On error, get_admhst returns KFAILURE. If all goes well, the routine
 * returns KSUCCESS.
 *
 * For the format of the KRB_CONF file, see comments describing the routine
 * krb_get_krbhst().
 *
 * This is a temporary hack to allow us to find the nearest system running
 * a Kerberos admin server.  In the long run, this functionality will be
 * provided by a nameserver.
 */

static int
krb_get_admhst_f(char *h, char *r, int n, const char *fname)
{
  FILE *cnffile;
  char tr[REALM_SZ];
  char linebuf[BUFSIZ];
  char scratch[64];
  register int i;

  if ((cnffile = fopen(fname,"r")) == NULL)
    return(KFAILURE);

  if (fgets(linebuf, BUFSIZ, cnffile) == NULL) {
    /* error reading */
    (void) fclose(cnffile);
    return(KFAILURE);
  }
  if (!strchr(linebuf, '\n')) {
    /* didn't all fit into buffer, punt */
    (void) fclose(cnffile);
    return(KFAILURE);
  }
  for (i = 0; i < n; ) {
    /* run through the file, looking for admin host */
    if (fgets(linebuf, BUFSIZ, cnffile) == NULL) {
      (void) fclose(cnffile);
      return(KFAILURE);
    }
    /* need to scan for a token after 'admin' to make sure that
       admin matched correctly */
    if (sscanf(linebuf, "%s %s admin %s", tr, h, scratch) != 3)
      continue;
    if (!strcmp(tr,r))
      i++;
  }
  (void) fclose(cnffile);
  return(KSUCCESS);
}

int
krb_get_admhst(char *h, char *r, int n)
{
  static const char *const files[] = KRB_RLM_FILES;
  int i;
  
  const char *dir = getenv("KRBCONFDIR");

  /* First try user specified file */
  if (dir != 0)
    {
      char fname[MaxPathLen];
      strncpy(fname, dir, sizeof(fname));
      strncat(fname, "/krb.realms", sizeof(fname));
      if (krb_get_admhst_f(h, r, n, fname) == KSUCCESS)
	return KSUCCESS;
    }

  for (i = 0; files[i] != 0; i++)
    if (krb_get_admhst_f(h, r, n, files[i]) == KSUCCESS)
      return KSUCCESS;

  /* If nothing else works try kerberos.REALMNAME if it exists */
  strncpy(h, "kerberos.", MaxHostNameLen);
  strncat(h, r, MaxHostNameLen);
  if (gethostbyname(h) != 0)
    return KSUCCESS;
  else
    return KFAILURE;
}

==> krb4-0.1.tar.gz: krb4-0.1/src/lib/krb/get_admhst.c <==
/*
 * This software may now be redistributed outside the US.
 *
 * $Source: /afs/nada.kth.se/home/staff/bg/work/krb4-0.1/src/lib/krb/RCS/get_admhst.c,v $
 *
 * $Locker:  $
 */

/* 
  Copyright (C) 1989 by the Massachusetts Institute of Technology

   Export of this software from the United States of America is assumed
   to require a specific license from the United States Government.
   It is the responsibility of any person or organization contemplating
   export to obtain such a license before exporting.

WITHIN THAT CONSTRAINT, permission to use, copy, modify, and
distribute this software and its documentation for any purpose and
without fee is hereby granted, provided that the above copyright
notice appear in all copies and that both that copyright notice and
this permission notice appear in supporting documentation, and that
the name of M.I.T. not be used in advertising or publicity pertaining
to distribution of the software without specific, written prior
permission.  M.I.T. makes no representations about the suitability of
this software for any purpose.  It is provided "as is" without express
or implied warranty.

  */

/*
 *
 * $Log: get_admhst.c,v $
 * Revision 1.5  1995/08/18  19:54:11  d91-jda
 * Changed a krb.realms that should have been krb.conf
 *
 * Revision 1.4  1995/08/18  17:04:14  bg
 * Was using krb.relms files instead of krb.conf files.
 *
 * Revision 1.3  1995/08/10 20:06:17  bg
 * Check if kerberos.REALMNAME server exist in the DNS.
 *
 * Revision 1.2  1995/08/10 13:35:04  bg
 * Use multiple configuration directories.
 *
 * Revision 1.1  1995/08/10 12:15:00  bg
 * Initial revision
 *
 *
 */

#include "krb_locl.h"

/*
 * Given a Kerberos realm, find a host on which the Kerberos database
 * administration server can be found.
 *
 * krb_get_admhst takes a pointer to be filled in, a pointer to the name
 * of the realm for which a server is desired, and an integer n, and
 * returns (in h) the nth administrative host entry from the configuration
 * file (KRB_CONF, defined in "krb.h") associated with the specified realm.
 *
 * On error, get_admhst returns KFAILURE. If all goes well, the routine
 * returns KSUCCESS.
 *
 * For the format of the KRB_CONF file, see comments describing the routine
 * krb_get_krbhst().
 *
 * This is a temporary hack to allow us to find the nearest system running
 * a Kerberos admin server.  In the long run, this functionality will be
 * provided by a nameserver.
 */

static int
krb_get_admhst_f(char *h, char *r, int n, const char *fname)
{
  FILE *cnffile;
  char tr[REALM_SZ];
  char linebuf[BUFSIZ];
  char scratch[64];
  register int i;

  if ((cnffile = fopen(fname,"r")) == NULL)
    return(KFAILURE);

  if (fgets(linebuf, BUFSIZ, cnffile) == NULL) {
    /* error reading */
    (void) fclose(cnffile);
    return(KFAILURE);
  }
  if (!strchr(linebuf, '\n')) {
    /* didn't all fit into buffer, punt */
    (void) fclose(cnffile);
    return(KFAILURE);
  }
  for (i = 0; i < n; ) {
    /* run through the file, looking for admin host */
    if (fgets(linebuf, BUFSIZ, cnffile) == NULL) {
      (void) fclose(cnffile);
      return(KFAILURE);
    }
    /* need to scan for a token after 'admin' to make sure that
       admin matched correctly */
    if (sscanf(linebuf, "%s %s admin %s", tr, h, scratch) != 3)
      continue;
    if (!strcmp(tr,r))
      i++;
  }
  (void) fclose(cnffile);
  return(KSUCCESS);
}

int
krb_get_admhst(char *h, char *r, int n)
{
  static const char *const files[] = KRB_CNF_FILES;
  int i;
  
  const char *dir = getenv("KRBCONFDIR");

  /* First try user specified file */
  if (dir != 0)
    {
      char fname[MaxPathLen];
      strncpy(fname, dir, sizeof(fname));
      strncat(fname, "/krb.conf", sizeof(fname));
      if (krb_get_admhst_f(h, r, n, fname) == KSUCCESS)
	return KSUCCESS;
    }

  for (i = 0; files[i] != 0; i++)
    if (krb_get_admhst_f(h, r, n, files[i]) == KSUCCESS)
      return KSUCCESS;

  /* If nothing else works try kerberos.REALMNAME if it exists */
  strncpy(h, "kerberos.", MaxHostNameLen);
  strncat(h, r, MaxHostNameLen);
  if (gethostbyname(h) != 0)
    return KSUCCESS;
  else
    return KFAILURE;
}
